# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Sharjahsat1(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field identifier: ax25_frame.payload.ax25_info.identifier
    :field tm_id: ax25_frame.payload.ax25_info.tm_id
    :field data_length: ax25_frame.payload.ax25_info.data_length
    :field packet_counter: ax25_frame.payload.ax25_info.packet_counter
    :field image_data: ax25_frame.payload.ax25_info.data.image_data.data.data_str
    :field op_mode: ax25_frame.payload.ax25_info.data.system_info.op_mode
    :field restart_count: ax25_frame.payload.ax25_info.data.system_info.restart_count
    :field last_reset_cause: ax25_frame.payload.ax25_info.data.system_info.last_reset_cause
    :field system_uptime: ax25_frame.payload.ax25_info.data.system_info.system_uptime
    :field system_time: ax25_frame.payload.ax25_info.data.system_info.system_time
    :field tlm_board_temp1: ax25_frame.payload.ax25_info.data.obc.tlm_board_temp1
    :field tlm_board_temp2: ax25_frame.payload.ax25_info.data.obc.tlm_board_temp2
    :field tlm_board_temp3: ax25_frame.payload.ax25_info.data.obc.tlm_board_temp3
    :field tlm_vbat_v: ax25_frame.payload.ax25_info.data.obc.tlm_vbat_v
    :field tlm_vbat_i: ax25_frame.payload.ax25_info.data.obc.tlm_vbat_i
    :field tlm_vbat_plat_v: ax25_frame.payload.ax25_info.data.obc.tlm_vbat_plat_v
    :field tlm_3v3_plat_v: ax25_frame.payload.ax25_info.data.obc.tlm_3v3_plat_v
    :field tlm_vbat_periph_i: ax25_frame.payload.ax25_info.data.obc.tlm_vbat_periph_i
    :field tlm_3v3_periph_i: ax25_frame.payload.ax25_info.data.obc.tlm_3v3_periph_i
    :field tlm_vbat_periph_v: ax25_frame.payload.ax25_info.data.obc.tlm_vbat_periph_v
    :field tlm_3v3_periph_v: ax25_frame.payload.ax25_info.data.obc.tlm_3v3_periph_v
    :field timestamp_hh: ax25_frame.payload.ax25_info.data.interfacebrd_rtc.timestamp_hh
    :field timestamp_mm: ax25_frame.payload.ax25_info.data.interfacebrd_rtc.timestamp_mm
    :field timestamp_ss: ax25_frame.payload.ax25_info.data.interfacebrd_rtc.timestamp_ss
    :field timestamp_dd: ax25_frame.payload.ax25_info.data.interfacebrd_rtc.timestamp_dd
    :field timestamp_mo: ax25_frame.payload.ax25_info.data.interfacebrd_rtc.timestamp_mo
    :field timestamp_yy: ax25_frame.payload.ax25_info.data.interfacebrd_rtc.timestamp_yy
    :field timestamp_dow: ax25_frame.payload.ax25_info.data.interfacebrd_rtc.timestamp_dow
    :field temperature: ax25_frame.payload.ax25_info.data.interfacebrd_rtc.temperature
    :field antenna_status: ax25_frame.payload.ax25_info.data.interfacebrd_rtc.antenna_status
    :field vbat: ax25_frame.payload.ax25_info.data.battery.vbat
    :field ibat: ax25_frame.payload.ax25_info.data.battery.ibat
    :field vpcm3v3: ax25_frame.payload.ax25_info.data.battery.vpcm3v3
    :field vpcm5v: ax25_frame.payload.ax25_info.data.battery.vpcm5v
    :field ipcm3v3: ax25_frame.payload.ax25_info.data.battery.ipcm3v3
    :field ipcm5v: ax25_frame.payload.ax25_info.data.battery.ipcm5v
    :field tbrd: ax25_frame.payload.ax25_info.data.battery.tbrd
    :field tbat1: ax25_frame.payload.ax25_info.data.battery.tbat1
    :field tbat2: ax25_frame.payload.ax25_info.data.battery.tbat2
    :field tbat3: ax25_frame.payload.ax25_info.data.battery.tbat3
    :field vpcmbatv: ax25_frame.payload.ax25_info.data.eps.vpcmbatv
    :field ipcmbatv: ax25_frame.payload.ax25_info.data.eps.ipcmbatv
    :field eps_vpcm3v3: ax25_frame.payload.ax25_info.data.eps.vpcm3v3
    :field eps_ipcm3v3: ax25_frame.payload.ax25_info.data.eps.ipcm3v3
    :field eps_vpcm5v: ax25_frame.payload.ax25_info.data.eps.vpcm5v
    :field eps_ipcm5v: ax25_frame.payload.ax25_info.data.eps.ipcm5v
    :field i3v3drw: ax25_frame.payload.ax25_info.data.eps.i3v3drw
    :field i5vdrw: ax25_frame.payload.ax25_info.data.eps.i5vdrw
    :field eps_tbrd: ax25_frame.payload.ax25_info.data.eps.tbrd
    :field tbrd_db: ax25_frame.payload.ax25_info.data.eps.tbrd_db
    :field ipcm12v: ax25_frame.payload.ax25_info.data.eps.ipcm12v
    :field vpcm12v: ax25_frame.payload.ax25_info.data.eps.vpcm12v
    :field adcs_state: ax25_frame.payload.ax25_info.data.adcs.adcs_state
    :field sat_pos_llh_lat: ax25_frame.payload.ax25_info.data.adcs.sat_pos_llh_lat
    :field sat_pos_llh_lon: ax25_frame.payload.ax25_info.data.adcs.sat_pos_llh_lon
    :field sat_pos_llh_alt: ax25_frame.payload.ax25_info.data.adcs.sat_pos_llh_alt
    :field estm_att_angle_yaw: ax25_frame.payload.ax25_info.data.adcs.estm_att_angle_yaw
    :field estm_att_angle_pitch: ax25_frame.payload.ax25_info.data.adcs.estm_att_angle_pitch
    :field estm_att_angle_roll: ax25_frame.payload.ax25_info.data.adcs.estm_att_angle_roll
    :field estm_ang_rate_yaw: ax25_frame.payload.ax25_info.data.adcs.estm_ang_rate_yaw
    :field estm_ang_rate_pitch: ax25_frame.payload.ax25_info.data.adcs.estm_ang_rate_pitch
    :field estm_ang_rate_roll: ax25_frame.payload.ax25_info.data.adcs.estm_ang_rate_roll
    :field adcs_gps: ax25_frame.payload.ax25_info.data.adcs.gps.data.data_str
    :field smps_temp: ax25_frame.payload.ax25_info.data.uhf_vhf_modem.smps_temp
    :field pa_temp: ax25_frame.payload.ax25_info.data.uhf_vhf_modem.pa_temp
    :field current_3v3: ax25_frame.payload.ax25_info.data.uhf_vhf_modem.current_3v3
    :field voltage_3v3: ax25_frame.payload.ax25_info.data.uhf_vhf_modem.voltage_3v3
    :field current_5v: ax25_frame.payload.ax25_info.data.uhf_vhf_modem.current_5v
    :field voltage_5v: ax25_frame.payload.ax25_info.data.uhf_vhf_modem.voltage_5v
    :field battery_current: ax25_frame.payload.ax25_info.data.s_band_modem.battery_current
    :field pa_current: ax25_frame.payload.ax25_info.data.s_band_modem.pa_current
    :field battery_voltage: ax25_frame.payload.ax25_info.data.s_band_modem.battery_voltage
    :field pa_voltage: ax25_frame.payload.ax25_info.data.s_band_modem.pa_voltage
    :field pa_temperature: ax25_frame.payload.ax25_info.data.s_band_modem.pa_temperature
    :field rf_output_power: ax25_frame.payload.ax25_info.data.s_band_modem.rf_output_power
    :field board_temp_top: ax25_frame.payload.ax25_info.data.s_band_modem.board_temp_top
    :field board_temp_bottom: ax25_frame.payload.ax25_info.data.s_band_modem.board_temp_bottom
    :field vbcr1: ax25_frame.payload.ax25_info.data.solar_panels.vbcr1
    :field vbcr2: ax25_frame.payload.ax25_info.data.solar_panels.vbcr2
    :field vbcr3: ax25_frame.payload.ax25_info.data.solar_panels.vbcr3
    :field vbcr4: ax25_frame.payload.ax25_info.data.solar_panels.vbcr4
    :field vbcr5: ax25_frame.payload.ax25_info.data.solar_panels.vbcr5
    :field vbcr6: ax25_frame.payload.ax25_info.data.solar_panels.vbcr6
    :field vbcr7: ax25_frame.payload.ax25_info.data.solar_panels.vbcr7
    :field vbcr8: ax25_frame.payload.ax25_info.data.solar_panels.vbcr8
    :field vbcr9: ax25_frame.payload.ax25_info.data.solar_panels.vbcr9
    :field ibcra1: ax25_frame.payload.ax25_info.data.solar_panels.ibcra1
    :field ibcra2: ax25_frame.payload.ax25_info.data.solar_panels.ibcra2
    :field ibcra3: ax25_frame.payload.ax25_info.data.solar_panels.ibcra3
    :field ibcra4: ax25_frame.payload.ax25_info.data.solar_panels.ibcra4
    :field ibcra5: ax25_frame.payload.ax25_info.data.solar_panels.ibcra5
    :field ibcra6: ax25_frame.payload.ax25_info.data.solar_panels.ibcra6
    :field ibcra7: ax25_frame.payload.ax25_info.data.solar_panels.ibcra7
    :field ibcra8: ax25_frame.payload.ax25_info.data.solar_panels.ibcra8
    :field ibcra9: ax25_frame.payload.ax25_info.data.solar_panels.ibcra9
    :field ibcrb1: ax25_frame.payload.ax25_info.data.solar_panels.ibcrb1
    :field ibcrb2: ax25_frame.payload.ax25_info.data.solar_panels.ibcrb2
    :field ibcrb3: ax25_frame.payload.ax25_info.data.solar_panels.ibcrb3
    :field ibcrb4: ax25_frame.payload.ax25_info.data.solar_panels.ibcrb4
    :field ibcrb5: ax25_frame.payload.ax25_info.data.solar_panels.ibcrb5
    :field ibcrb6: ax25_frame.payload.ax25_info.data.solar_panels.ibcrb6
    :field ibcrb7: ax25_frame.payload.ax25_info.data.solar_panels.ibcrb7
    :field ibcrb8: ax25_frame.payload.ax25_info.data.solar_panels.ibcrb8
    :field ibcrb9: ax25_frame.payload.ax25_info.data.solar_panels.ibcrb9
    :field tbcra1: ax25_frame.payload.ax25_info.data.solar_panels.tbcra1
    :field tbcra2: ax25_frame.payload.ax25_info.data.solar_panels.tbcra2
    :field tbcra3: ax25_frame.payload.ax25_info.data.solar_panels.tbcra3
    :field tbcra4: ax25_frame.payload.ax25_info.data.solar_panels.tbcra4
    :field tbcra5: ax25_frame.payload.ax25_info.data.solar_panels.tbcra5
    :field tbcra6: ax25_frame.payload.ax25_info.data.solar_panels.tbcra6
    :field tbcra7: ax25_frame.payload.ax25_info.data.solar_panels.tbcra7
    :field tbcra8: ax25_frame.payload.ax25_info.data.solar_panels.tbcra8
    :field tbcra9: ax25_frame.payload.ax25_info.data.solar_panels.tbcra9
    :field tbcrb1: ax25_frame.payload.ax25_info.data.solar_panels.tbcrb1
    :field tbcrb2: ax25_frame.payload.ax25_info.data.solar_panels.tbcrb2
    :field tbcrb3: ax25_frame.payload.ax25_info.data.solar_panels.tbcrb3
    :field tbcrb4: ax25_frame.payload.ax25_info.data.solar_panels.tbcrb4
    :field tbcrb5: ax25_frame.payload.ax25_info.data.solar_panels.tbcrb5
    :field tbcrb6: ax25_frame.payload.ax25_info.data.solar_panels.tbcrb6
    :field tbcrb7: ax25_frame.payload.ax25_info.data.solar_panels.tbcrb7
    :field tbcrb8: ax25_frame.payload.ax25_info.data.solar_panels.tbcrb8
    :field tbcrb9: ax25_frame.payload.ax25_info.data.solar_panels.tbcrb9
    :field vidiodeout: ax25_frame.payload.ax25_info.data.solar_panels.vidiodeout
    :field iidiodeout: ax25_frame.payload.ax25_info.data.solar_panels.iidiodeout
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Sharjahsat1.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Sharjahsat1.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Sharjahsat1.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Sharjahsat1.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Sharjahsat1.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Sharjahsat1.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Sharjahsat1.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Sharjahsat1.IFrame(self._io, self, self._root)


    class SystemInfoT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.op_mode = self._io.read_u2le()
            self.restart_count = self._io.read_u2le()
            self.last_reset_cause = self._io.read_u1()
            self.system_uptime = self._io.read_u4le()
            self.system_time = self._io.read_u4le()


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Sharjahsat1.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Sharjahsat1.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Sharjahsat1.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Sharjahsat1.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Sharjahsat1.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Sharjahsat1.HeaderT(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"A60UOS") or (self.callsign == u"A62UOS")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class ImageT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw_image_data = self._io.read_bytes_full()
            _io__raw_image_data = KaitaiStream(BytesIO(self._raw_image_data))
            self.image_data = Sharjahsat1.DataB64T(_io__raw_image_data, self, self._root)


    class StrB64T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data_str = (self._io.read_bytes_full()).decode(u"ASCII")


    class EpsT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.vpcmbatv = self._io.read_u2le()
            self.ipcmbatv = self._io.read_s2le()
            self.vpcm3v3 = self._io.read_u2le()
            self.ipcm3v3 = self._io.read_u2le()
            self.vpcm5v = self._io.read_u2le()
            self.ipcm5v = self._io.read_u2le()
            self.i3v3drw = self._io.read_u2le()
            self.i5vdrw = self._io.read_u2le()
            self.tbrd = self._io.read_u2le()
            self.tbrd_db = self._io.read_u2le()
            self.ipcm12v = self._io.read_u2le()
            self.vpcm12v = self._io.read_u2le()


    class DataB64T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_data = self._io.read_bytes_full()
            _process = satnogsdecoders.process.B64encode()
            self._raw_data = _process.decode(self._raw__raw_data)
            _io__raw_data = KaitaiStream(BytesIO(self._raw_data))
            self.data = Sharjahsat1.StrB64T(_io__raw_data, self, self._root)


    class AdcsT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.adcs_state = self._io.read_u1()
            self.sat_pos_llh_lat = self._io.read_s2le()
            self.sat_pos_llh_lon = self._io.read_s2le()
            self.sat_pos_llh_alt = self._io.read_s2le()
            self.estm_att_angle_yaw = self._io.read_s2le()
            self.estm_att_angle_pitch = self._io.read_s2le()
            self.estm_att_angle_roll = self._io.read_s2le()
            self.estm_ang_rate_yaw = self._io.read_s2le()
            self.estm_ang_rate_pitch = self._io.read_s2le()
            self.estm_ang_rate_roll = self._io.read_s2le()
            self._raw_gps = self._io.read_bytes(18)
            _io__raw_gps = KaitaiStream(BytesIO(self._raw_gps))
            self.gps = Sharjahsat1.DataB64T(_io__raw_gps, self, self._root)


    class HeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.identifier = (self._io.read_bytes(4)).decode(u"ASCII")
            if not  ((self.identifier == u"ESER")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.identifier, self._io, u"/types/header_t/seq/0")
            self.tm_id = self._io.read_u1()
            self.data_length = self._io.read_u1()
            self.packet_counter = self._io.read_u4le()
            _on = self.tm_id
            if _on == 65:
                self.data = Sharjahsat1.ImageT(self._io, self, self._root)
            elif _on == 80:
                self.data = Sharjahsat1.TlmT(self._io, self, self._root)


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class UhfVhfModemT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.smps_temp = self._io.read_u1()
            self.pa_temp = self._io.read_u1()
            self.current_3v3 = self._io.read_u2le()
            self.voltage_3v3 = self._io.read_u2le()
            self.current_5v = self._io.read_u2le()
            self.voltage_5v = self._io.read_u2le()


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Sharjahsat1.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Sharjahsat1.SsidMask(self._io, self, self._root)


    class InterfacebrdRtcT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp_hh = self._io.read_u1()
            self.timestamp_mm = self._io.read_u1()
            self.timestamp_ss = self._io.read_u1()
            self.timestamp_dd = self._io.read_u1()
            self.timestamp_mo = self._io.read_u1()
            self.timestamp_yy = self._io.read_u1()
            self.timestamp_dow = self._io.read_u1()
            self.temperature = self._io.read_s2le()
            self.antenna_status = self._io.read_u1()


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Sharjahsat1.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class SolarPanelsT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.vbcr1 = self._io.read_u2le()
            self.vbcr2 = self._io.read_u2le()
            self.vbcr3 = self._io.read_u2le()
            self.vbcr4 = self._io.read_u2le()
            self.vbcr5 = self._io.read_u2le()
            self.vbcr6 = self._io.read_u2le()
            self.vbcr7 = self._io.read_u2le()
            self.vbcr8 = self._io.read_u2le()
            self.vbcr9 = self._io.read_u2le()
            self.ibcra1 = self._io.read_u2le()
            self.ibcra2 = self._io.read_u2le()
            self.ibcra3 = self._io.read_u2le()
            self.ibcra4 = self._io.read_u2le()
            self.ibcra5 = self._io.read_u2le()
            self.ibcra6 = self._io.read_u2le()
            self.ibcra7 = self._io.read_u2le()
            self.ibcra8 = self._io.read_u2le()
            self.ibcra9 = self._io.read_u2le()
            self.ibcrb1 = self._io.read_u2le()
            self.ibcrb2 = self._io.read_u2le()
            self.ibcrb3 = self._io.read_u2le()
            self.ibcrb4 = self._io.read_u2le()
            self.ibcrb5 = self._io.read_u2le()
            self.ibcrb6 = self._io.read_u2le()
            self.ibcrb7 = self._io.read_u2le()
            self.ibcrb8 = self._io.read_u2le()
            self.ibcrb9 = self._io.read_u2le()
            self.tbcra1 = self._io.read_s2le()
            self.tbcra2 = self._io.read_s2le()
            self.tbcra3 = self._io.read_s2le()
            self.tbcra4 = self._io.read_s2le()
            self.tbcra5 = self._io.read_s2le()
            self.tbcra6 = self._io.read_s2le()
            self.tbcra7 = self._io.read_s2le()
            self.tbcra8 = self._io.read_s2le()
            self.tbcra9 = self._io.read_s2le()
            self.tbcrb1 = self._io.read_s2le()
            self.tbcrb2 = self._io.read_s2le()
            self.tbcrb3 = self._io.read_s2le()
            self.tbcrb4 = self._io.read_s2le()
            self.tbcrb5 = self._io.read_s2le()
            self.tbcrb6 = self._io.read_s2le()
            self.tbcrb7 = self._io.read_s2le()
            self.tbcrb8 = self._io.read_s2le()
            self.tbcrb9 = self._io.read_s2le()
            self.vidiodeout = self._io.read_u2le()
            self.iidiodeout = self._io.read_u2le()


    class SBandModemT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.battery_current = self._io.read_u2le()
            self.pa_current = self._io.read_u2le()
            self.battery_voltage = self._io.read_u2le()
            self.pa_voltage = self._io.read_u2le()
            self.pa_temperature = self._io.read_u2le()
            self.rf_output_power = self._io.read_u2le()
            self.board_temp_top = self._io.read_s2le()
            self.board_temp_bottom = self._io.read_s2le()


    class ObcT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.tlm_board_temp1 = self._io.read_s2le()
            self.tlm_board_temp2 = self._io.read_s2le()
            self.tlm_board_temp3 = self._io.read_s2le()
            self.tlm_vbat_v = self._io.read_u2le()
            self.tlm_vbat_i = self._io.read_u2le()
            self.tlm_vbat_plat_v = self._io.read_u2le()
            self.tlm_3v3_plat_v = self._io.read_u2le()
            self.tlm_vbat_periph_i = self._io.read_u2le()
            self.tlm_3v3_periph_i = self._io.read_u2le()
            self.tlm_vbat_periph_v = self._io.read_u2le()
            self.tlm_3v3_periph_v = self._io.read_u2le()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Sharjahsat1.Callsign(_io__raw_callsign_ror, self, self._root)


    class TlmT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.system_info = Sharjahsat1.SystemInfoT(self._io, self, self._root)
            self.obc = Sharjahsat1.ObcT(self._io, self, self._root)
            self.interfacebrd_rtc = Sharjahsat1.InterfacebrdRtcT(self._io, self, self._root)
            self.battery = Sharjahsat1.BatteryT(self._io, self, self._root)
            self.eps = Sharjahsat1.EpsT(self._io, self, self._root)
            self.adcs = Sharjahsat1.AdcsT(self._io, self, self._root)
            self.uhf_vhf_modem = Sharjahsat1.UhfVhfModemT(self._io, self, self._root)
            self.s_band_modem = Sharjahsat1.SBandModemT(self._io, self, self._root)
            self.solar_panels = Sharjahsat1.SolarPanelsT(self._io, self, self._root)


    class BatteryT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.vbat = self._io.read_u2le()
            self.ibat = self._io.read_s2le()
            self.vpcm3v3 = self._io.read_u2le()
            self.vpcm5v = self._io.read_u2le()
            self.ipcm3v3 = self._io.read_u2le()
            self.ipcm5v = self._io.read_u2le()
            self.tbrd = self._io.read_s2le()
            self.tbat1 = self._io.read_s2le()
            self.tbat2 = self._io.read_s2le()
            self.tbat3 = self._io.read_s2le()






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Foresail1(KaitaiStruct):
    """:field timestamp: pus.obc_housekeeping.timestamp.timestamp
    :field obc_side: pus.obc_housekeeping.side
    :field obc_fdir: pus.obc_housekeeping.fdir
    :field obc_scheduler: pus.obc_housekeeping.scheduler
    :field obc_sw_revision: pus.obc_housekeeping.software_revision
    :field obc_uptime: pus.obc_housekeeping.uptime
    :field obc_heap_free: pus.obc_housekeeping.heap_free
    :field obc_cpu_load: pus.obc_housekeeping.cpu_load
    :field obc_fs_free_space: pus.obc_housekeeping.fs_free_space
    :field arbiter_uptime: pus.obc_housekeeping.arbiter_uptime
    :field arbiter_age: pus.obc_housekeeping.arbiter_age
    :field arbiter_bootcount: pus.obc_housekeeping.arbiter_bootcount
    :field arbiter_temperature: pus.obc_housekeeping.arbiter_temperature
    :field side_a_bootcount: pus.obc_housekeeping.side_a_bootcount
    :field side_a_heartbeat: pus.obc_housekeeping.side_a_heartbeat
    :field side_a_fail_counter: pus.obc_housekeeping.side_a_fail_counter
    :field side_a_fail_reason: pus.obc_housekeeping.side_a_fail_reason
    :field side_b_bootcount: pus.obc_housekeeping.side_b_bootcount
    :field side_b_heartbeat: pus.obc_housekeeping.side_b_heartbeat
    :field side_b_fail_counter: pus.obc_housekeeping.side_b_fail_counter
    :field side_b_fail_reason: pus.obc_housekeeping.side_b_fail_reason
    
    :field timestamp: pus.eps_housekeeping.timestamp.timestamp
    :field pcdu_uptime: pus.eps_housekeeping.pcdu_uptime
    :field pcdu_boot_count: pus.eps_housekeeping.pcdu_boot_count
    :field pdm_expected: pus.eps_housekeeping.pdm_expected
    :field pdm_faulted: pus.eps_housekeeping.pdm_faulted
    :field pcdu_peak_detect_index: pus.eps_housekeeping.pcdu_peak_detect_index
    :field panel_x_minus_voltage: pus.eps_housekeeping.v_in_x_minus
    :field panel_x_plus_voltage: pus.eps_housekeeping.v_in_x_plus
    :field panel_y_minus_voltage: pus.eps_housekeeping.v_in_y_minus
    :field panel_y_plus_voltage: pus.eps_housekeeping.v_in_y_plus
    :field panel_x_minus_max_voltage: pus.eps_housekeeping.v_in_max_x_minus
    :field panel_x_plus_max_voltage: pus.eps_housekeeping.v_in_max_x_plus
    :field panel_y_minus_max_voltage: pus.eps_housekeeping.v_in_max_y_minus
    :field panel_y_plus_max_voltage: pus.eps_housekeeping.v_in_max_y_plus
    :field panel_x_minus_current: pus.eps_housekeeping.i_in_x_minus
    :field panel_x_plus_current: pus.eps_housekeeping.i_in_x_plus
    :field panel_y_minus_current: pus.eps_housekeeping.i_in_y_minus
    :field panel_y_plus_current: pus.eps_housekeeping.i_in_y_plus
    :field panel_x_minus_max_current: pus.eps_housekeeping.i_in_max_x_minus
    :field panel_x_plus_max_current: pus.eps_housekeeping.i_in_max_x_plus
    :field panel_y_minus_max_current: pus.eps_housekeeping.i_in_max_y_minus
    :field panel_y_plus_max_current: pus.eps_housekeeping.i_in_max_y_plus
    :field v_batt_bus: pus.eps_housekeeping.v_batt_bus
    :field temp_x_minus: pus.eps_housekeeping.temp_x_minus
    :field temp_x_plus: pus.eps_housekeeping.temp_x_plus
    :field temp_y_minus: pus.eps_housekeeping.temp_y_minus
    :field temp_y_plus: pus.eps_housekeeping.temp_y_plus
    :field temp_pcdu: pus.eps_housekeeping.temp_pcdu
    :field v_3v6_uhd_adcs: pus.eps_housekeeping.v_3v6_uhd_adcs
    :field v_3v6_mag_obc: pus.eps_housekeeping.v_3v6_mag_obc
    :field v_3v6_epb_cam: pus.eps_housekeeping.v_3v6_epb_cam
    :field i_pate_batt: pus.eps_housekeeping.i_pate_batt
    :field pb_batt_current: pus.eps_housekeeping.i_pb_batt
    :field pb_3v6_current: pus.eps_housekeeping.i_pb_3v6
    :field cam_3v6_current: pus.eps_housekeeping.i_cam_3v6
    :field mag_3v6_current: pus.eps_housekeeping.i_mag_3v6
    :field obc_3v6_current: pus.eps_housekeeping.i_obc_3v6
    :field uhf_3v6_current: pus.eps_housekeeping.i_uhf_3v6
    :field adcs_3v6_current: pus.eps_housekeeping.i_adcs_3v6
    :field pate_batt_current_max: pus.eps_housekeeping.i_pate_batt_max
    :field pb_batt_current_max: pus.eps_housekeeping.i_pb_batt_max
    :field pb_3v6_current_max: pus.eps_housekeeping.i_pb_3v6_max
    :field cam_3v6_current_max: pus.eps_housekeeping.i_cam_3v6_max
    :field mag_3v6_current_max: pus.eps_housekeeping.i_mag_3v6_max
    :field obc_3v6_current_max: pus.eps_housekeeping.i_obc_3v6_max
    :field uhf_3v6_current_max: pus.eps_housekeeping.i_uhf_3v6_max
    :field adcs_3v6_current_max: pus.eps_housekeeping.i_adcs_3v6_max
    :field pate_batt_current_min: pus.eps_housekeeping.i_pate_batt_min
    :field pb_batt_current_min: pus.eps_housekeeping.i_pb_batt_min
    :field pb_3v6_current_min: pus.eps_housekeeping.i_pb_3v6_min
    :field cam_3v6_current_min: pus.eps_housekeeping.i_cam_3v6_min
    :field mag_3v6_current_min: pus.eps_housekeeping.i_mag_3v6_min
    :field obc_3v6_current_min: pus.eps_housekeeping.i_obc_3v6_min
    :field uhf_3v6_current_min: pus.eps_housekeeping.i_uhf_3v6_min
    :field adcs_3v6_current_min: pus.eps_housekeeping.i_adcs_3v6_min
    :field batt_status: pus.eps_housekeeping.batt_status
    :field batt_boot_count: pus.eps_housekeeping.batt_boot_count
    :field batt_wdt_reset_count: pus.eps_housekeeping.batt_wdt_reset_count
    :field batt_bus_timeout_count: pus.eps_housekeeping.batt_bus_timeout_count
    :field batt_bpc_fail_count: pus.eps_housekeeping.batt_bpc_fail_count
    :field batt_pack_voltage: pus.eps_housekeeping.batt_pack_voltage
    :field batt_pack_lower_voltage: pus.eps_housekeeping.batt_pack_lower_voltage
    :field batt_pack_current: pus.eps_housekeeping.batt_pack_current
    :field batt_pack_min_current: pus.eps_housekeeping.batt_pack_min_current
    :field batt_pack_max_current: pus.eps_housekeeping.batt_pack_max_current
    :field batt_pack_temp: pus.eps_housekeeping.batt_pack_temp
    :field batt_board_temp: pus.eps_housekeeping.batt_board_temp
    :field batt_heater_pwm_on_time: pus.eps_housekeeping.heater_pwm_on_time
    
    :field timestamp: pus.uhf_housekeeping.timestamp.timestamp
    :field uhf_uptime: pus.uhf_housekeeping.uptime
    :field uhf_bootcount: pus.uhf_housekeeping.bootcount
    :field uhf_wdt_resets: pus.uhf_housekeeping.wdt_reset_count
    :field uhf_sbe_count: pus.uhf_housekeeping.sbe_count
    :field uhf_mbe_count: pus.uhf_housekeeping.mbe_count
    :field uhf_total_tx_frames: pus.uhf_housekeeping.total_tx_frames
    :field uhf_total_rx_frames: pus.uhf_housekeeping.total_rx_frames
    :field uhf_total_ham_tx_frames: pus.uhf_housekeeping.total_ham_tx_frames
    :field uhf_total_ham_rx_frames: pus.uhf_housekeeping.total_ham_rx_frames
    :field uhf_side: pus.uhf_housekeeping.side
    :field uhf_rx_mode: pus.uhf_housekeeping.rx_mode
    :field uhf_tx_mode: pus.uhf_housekeeping.tx_mode
    :field uhf_mcu_temperature: pus.uhf_housekeeping.mcu_temperature
    :field uhf_pa_temperature: pus.uhf_housekeeping.pa_temperature
    :field uhf_background_rssi: pus.uhf_housekeeping.backogrund_rssi
    :field uhf_last_rssi: pus.uhf_housekeeping.last_rssi
    :field uhf_last_freq_offset: pus.uhf_housekeeping.last_freq_offset
    
    :field timestamp: pus.adcs_housekeeping.timestamp.timestamp
    :field determination_mode: pus.adcs_housekeeping.determination_mode
    :field control_mode: pus.adcs_housekeeping.control_mode
    :field mjd: pus.adcs_housekeeping.mjd
    :field position_x: pus.adcs_housekeeping.position_x
    :field position_y: pus.adcs_housekeeping.position_y
    :field position_z: pus.adcs_housekeeping.position_z
    :field velocity_x: pus.adcs_housekeeping.velocity_x
    :field velocity_y: pus.adcs_housekeeping.velocity_y
    :field velocity_z: pus.adcs_housekeeping.velocity_z
    :field quaternion_x: pus.adcs_housekeeping.quaternion_x
    :field quaternion_y: pus.adcs_housekeeping.quaternion_y
    :field quaternion_z: pus.adcs_housekeeping.quaternion_z
    :field quaternion_w: pus.adcs_housekeeping.quaternion_w
    :field angular_rate_x: pus.adcs_housekeeping.angular_rate_x
    :field angular_rate_y: pus.adcs_housekeeping.angular_rate_y
    :field angular_rate_z: pus.adcs_housekeeping.angular_rate_z
    
    :field timestamp: pus.event.timestamp.timestamp
    :field event_severity: pus.event.severity
    :field event_rid: pus.event.rid
    :field event_info: pus.event.info
    
    :field repeater_dest_callsign: repeater.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field repeater_dest_ssid: repeater.ax25_header.dest_ssid_raw.ssid
    :field repeater_src_callsign: repeater.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field repeater_src_ssid: repeater.ax25_header.src_ssid_raw.ssid
    :field repeater_payload: repeater.payload
    
    .. seealso::
       Source - https://foresail.github.io/docs/FS1_Space_Ground_Interface_Control_Sheet.pdf
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.skylink = Foresail1.SkylinkFrame(self._io, self, self._root)
        if  ((self.skylink.vc == 0) or (self.skylink.vc == 1)) :
            self._raw_pus = self._io.read_bytes(((self._io.size() - self._io.pos()) - (8 * self.skylink.is_authenticated)))
            _io__raw_pus = KaitaiStream(BytesIO(self._raw_pus))
            self.pus = Foresail1.ForesailPusFrame(_io__raw_pus, self, self._root)

        if self.skylink.vc == 3:
            self._raw_repeater = self._io.read_bytes(((self._io.size() - self._io.pos()) - (8 * self.skylink.is_authenticated)))
            _io__raw_repeater = KaitaiStream(BytesIO(self._raw_repeater))
            self.repeater = Foresail1.Ax25Frame(_io__raw_repeater, self, self._root)

        if self.skylink.is_authenticated == 1:
            self.auth = self._io.read_bytes(8)


    class Ax25Frame(KaitaiStruct):
        """
        .. seealso::
           Source - https://www.tapr.org/pdf/AX25.2.2.pdf
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.start_flag = self._io.read_bytes(1)
            if not self.start_flag == b"\x7E":
                raise kaitaistruct.ValidationNotEqualError(b"\x7E", self.start_flag, self._io, u"/types/ax25_frame/seq/0")
            self.ax25_header = Foresail1.Ax25Header(self._io, self, self._root)
            self.payload = (KaitaiStream.bytes_terminate(self._io.read_bytes((((self._io.size() - self._io.pos()) - 3) - (8 * self._root.skylink.is_authenticated))), 0, False)).decode(u"ascii")
            self.fcs = self._io.read_u2be()
            self.end_flag = self._io.read_bytes(1)
            if not self.end_flag == b"\x7E":
                raise kaitaistruct.ValidationNotEqualError(b"\x7E", self.end_flag, self._io, u"/types/ax25_frame/seq/4")


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Foresail1.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Foresail1.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Foresail1.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Foresail1.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()
            self.pid = self._io.read_u1()


    class PusHeader(KaitaiStruct):
        """CCSDS PUS header."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.packet_id = self._io.read_u2be()
            self.sequence = self._io.read_u2be()
            self.length = self._io.read_u2be()
            self.secondary_header = self._io.read_u1()
            self.service_type = self._io.read_u1()
            self.service_subtype = self._io.read_u1()


    class AdcsHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = Foresail1.UnixTimestamp(self._io, self, self._root)
            self.determination_mode = self._io.read_u1()
            self.control_mode = self._io.read_u1()
            self.mjd = self._io.read_f4le()
            self.position_x = self._io.read_f4le()
            self.position_y = self._io.read_f4le()
            self.position_z = self._io.read_f4le()
            self.velocity_x = self._io.read_f4le()
            self.velocity_y = self._io.read_f4le()
            self.velocity_z = self._io.read_f4le()
            self.angular_rate_x = self._io.read_f4le()
            self.angular_rate_y = self._io.read_f4le()
            self.angular_rate_z = self._io.read_f4le()
            self.quaternion_x = self._io.read_f4le()
            self.quaternion_y = self._io.read_f4le()
            self.quaternion_z = self._io.read_f4le()
            self.quaternion_w = self._io.read_f4le()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class UhfHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = Foresail1.UnixTimestamp(self._io, self, self._root)
            self.uptime = self._io.read_u4le()
            self.bootcount = self._io.read_u2le()
            self.wdt_reset_count = self._io.read_u1()
            self.sbe_count = self._io.read_u1()
            self.mbe_count = self._io.read_u1()
            self.bus_sync_errors = self._io.read_u1()
            self.bus_len_errors = self._io.read_u1()
            self.bus_crc_errors = self._io.read_u1()
            self.bus_bug_errors = self._io.read_u1()
            self.total_tx_frames = self._io.read_u4le()
            self.total_rx_frames = self._io.read_u4le()
            self.total_ham_tx_frames = self._io.read_u4le()
            self.total_ham_rx_frames = self._io.read_u4le()
            self.side = self._io.read_u1()
            self.rx_mode = self._io.read_u1()
            self.tx_mode = self._io.read_u1()
            self.mcu_temperature = self._io.read_s2le()
            self.pa_temperature = self._io.read_s2le()
            self.background_rssi = self._io.read_s1()
            self.last_rssi = self._io.read_s1()
            self.last_freq_offset = self._io.read_s2le()


    class UnixTimestamp(KaitaiStruct):
        """Unix timestamp."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = self._io.read_u4be()


    class Event(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = Foresail1.UnixTimestamp(self._io, self, self._root)
            self.rid = self._io.read_u2be()
            self.info = self._io.read_bytes_full()


    class EpsHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = Foresail1.UnixTimestamp(self._io, self, self._root)
            self.pcdu_uptime = self._io.read_u4le()
            self.pcdu_boot_count = self._io.read_u1()
            self.pdm_expected = self._io.read_u1()
            self.pdm_faulted = self._io.read_u1()
            self.pcdu_peak_detect_index = self._io.read_u1()
            self.v_in_x_minus = self._io.read_u2le()
            self.v_in_x_plus = self._io.read_u2le()
            self.v_in_y_minus = self._io.read_u2le()
            self.v_in_y_plus = self._io.read_u2le()
            self.v_in_max_x_minus = self._io.read_u2le()
            self.v_in_max_x_plus = self._io.read_u2le()
            self.v_in_max_y_minus = self._io.read_u2le()
            self.v_in_max_y_plus = self._io.read_u2le()
            self.i_in_x_minus = self._io.read_u2le()
            self.i_in_x_plus = self._io.read_u2le()
            self.i_in_y_minus = self._io.read_u2le()
            self.i_in_y_plus = self._io.read_u2le()
            self.i_in_max_x_minus = self._io.read_u2le()
            self.i_in_max_x_plus = self._io.read_u2le()
            self.i_in_max_y_minus = self._io.read_u2le()
            self.i_in_max_y_plus = self._io.read_u2le()
            self.v_batt_bus = self._io.read_u2le()
            self.temp_x_minus = self._io.read_s2le()
            self.temp_x_plus = self._io.read_s2le()
            self.temp_y_minus = self._io.read_s2le()
            self.temp_y_plus = self._io.read_s2le()
            self.temp_pcdu = self._io.read_s2le()
            self.v_3v6_uhd_adcs = self._io.read_u2le()
            self.v_3v6_mag_obc = self._io.read_u2le()
            self.v_3v6_epb_cam = self._io.read_u2le()
            self.i_pate_batt = self._io.read_u2le()
            self.i_pb_batt = self._io.read_u2le()
            self.i_pb_3v6 = self._io.read_u2le()
            self.i_cam_3v6 = self._io.read_u2le()
            self.i_mag_3v6 = self._io.read_u2le()
            self.i_obc_3v6 = self._io.read_u2le()
            self.i_uhf_3v6 = self._io.read_u2le()
            self.i_adcs_3v6 = self._io.read_u2le()
            self.i_pate_batt_max = self._io.read_u2le()
            self.i_pb_batt_max = self._io.read_u2le()
            self.i_pb_3v6_max = self._io.read_u2le()
            self.i_cam_3v6_max = self._io.read_u2le()
            self.i_mag_3v6_max = self._io.read_u2le()
            self.i_obc_3v6_max = self._io.read_u2le()
            self.i_uhf_3v6_max = self._io.read_u2le()
            self.i_adcs_3v6_max = self._io.read_u2le()
            self.i_pate_batt_min = self._io.read_u2le()
            self.i_pb_batt_min = self._io.read_u2le()
            self.i_pb_3v6_min = self._io.read_u2le()
            self.i_cam_3v6_min = self._io.read_u2le()
            self.i_mag_3v6_min = self._io.read_u2le()
            self.i_obc_3v6_min = self._io.read_u2le()
            self.i_uhf_3v6_min = self._io.read_u2le()
            self.i_adcs_3v6_min = self._io.read_u2le()
            self.batt_status = self._io.read_u2le()
            self.batt_boot_count = self._io.read_u1()
            self.batt_wdt_reset_count = self._io.read_u1()
            self.batt_bus_timeout_count = self._io.read_u1()
            self.batt_bpc_fail_count = self._io.read_u1()
            self.batt_pack_voltage = self._io.read_u2le()
            self.batt_pack_lower_voltage = self._io.read_u2le()
            self.batt_pack_current = self._io.read_s2le()
            self.batt_pack_min_current = self._io.read_s2le()
            self.batt_pack_max_current = self._io.read_s2le()
            self.batt_pack_temp = self._io.read_s2le()
            self.batt_board_temp = self._io.read_s2le()
            self.heater_pwm_on_time = self._io.read_u2le()


    class ForesailPusFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = Foresail1.PusHeader(self._io, self, self._root)
            if  ((self.header.service_type == 3) and (self.header.service_subtype == 2)) :
                self.obc_housekeeping = Foresail1.ObcHousekeeping(self._io, self, self._root)

            if  ((self.header.service_type == 3) and (self.header.service_subtype == 3)) :
                self.eps_housekeeping = Foresail1.EpsHousekeeping(self._io, self, self._root)

            if  ((self.header.service_type == 3) and (self.header.service_subtype == 4)) :
                self.uhf_housekeeping = Foresail1.UhfHousekeeping(self._io, self, self._root)

            if  ((self.header.service_type == 3) and (self.header.service_subtype == 5)) :
                self.adcs_housekeeping = Foresail1.AdcsHousekeeping(self._io, self, self._root)

            if  ((self.header.service_type == 4) and ( ((self.header.service_subtype == 1) or (self.header.service_subtype == 2) or (self.header.service_subtype == 3) or (self.header.service_subtype == 4)) )) :
                self.event = Foresail1.Event(self._io, self, self._root)



    class Pdms(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.val = self._io.read_u1()

        @property
        def mag_3v6(self):
            if hasattr(self, '_m_mag_3v6'):
                return self._m_mag_3v6 if hasattr(self, '_m_mag_3v6') else None

            self._m_mag_3v6 = ((self.val & 16) >> 4)
            return self._m_mag_3v6 if hasattr(self, '_m_mag_3v6') else None

        @property
        def uhf_3v6(self):
            if hasattr(self, '_m_uhf_3v6'):
                return self._m_uhf_3v6 if hasattr(self, '_m_uhf_3v6') else None

            self._m_uhf_3v6 = ((self.val & 64) >> 6)
            return self._m_uhf_3v6 if hasattr(self, '_m_uhf_3v6') else None

        @property
        def pate_batt(self):
            if hasattr(self, '_m_pate_batt'):
                return self._m_pate_batt if hasattr(self, '_m_pate_batt') else None

            self._m_pate_batt = ((self.val & 1) >> 0)
            return self._m_pate_batt if hasattr(self, '_m_pate_batt') else None

        @property
        def obc_3v6(self):
            if hasattr(self, '_m_obc_3v6'):
                return self._m_obc_3v6 if hasattr(self, '_m_obc_3v6') else None

            self._m_obc_3v6 = ((self.val & 32) >> 5)
            return self._m_obc_3v6 if hasattr(self, '_m_obc_3v6') else None

        @property
        def cam_3v6(self):
            if hasattr(self, '_m_cam_3v6'):
                return self._m_cam_3v6 if hasattr(self, '_m_cam_3v6') else None

            self._m_cam_3v6 = ((self.val & 8) >> 3)
            return self._m_cam_3v6 if hasattr(self, '_m_cam_3v6') else None

        @property
        def pb_3v6(self):
            if hasattr(self, '_m_pb_3v6'):
                return self._m_pb_3v6 if hasattr(self, '_m_pb_3v6') else None

            self._m_pb_3v6 = ((self.val & 4) >> 2)
            return self._m_pb_3v6 if hasattr(self, '_m_pb_3v6') else None

        @property
        def adcs_3v6(self):
            if hasattr(self, '_m_adcs_3v6'):
                return self._m_adcs_3v6 if hasattr(self, '_m_adcs_3v6') else None

            self._m_adcs_3v6 = ((self.val & 128) >> 7)
            return self._m_adcs_3v6 if hasattr(self, '_m_adcs_3v6') else None

        @property
        def pb_batt(self):
            if hasattr(self, '_m_pb_batt'):
                return self._m_pb_batt if hasattr(self, '_m_pb_batt') else None

            self._m_pb_batt = ((self.val & 2) >> 1)
            return self._m_pb_batt if hasattr(self, '_m_pb_batt') else None


    class SkylinkFrame(KaitaiStruct):
        """
        .. seealso::
           Skylink Protocol Specification.pdf
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.start_byte = self._io.read_bytes(1)
            if not self.start_byte == b"\x66":
                raise kaitaistruct.ValidationNotEqualError(b"\x66", self.start_byte, self._io, u"/types/skylink_frame/seq/0")
            self.identity = self._io.read_bytes(6)
            if not self.identity == b"\x4F\x48\x32\x46\x31\x53":
                raise kaitaistruct.ValidationNotEqualError(b"\x4F\x48\x32\x46\x31\x53", self.identity, self._io, u"/types/skylink_frame/seq/1")
            self.control = self._io.read_u1()
            self.extension_length = self._io.read_u1()
            self.sequence_number = self._io.read_u2be()
            self.extensions = self._io.read_bytes(self.extension_length)

        @property
        def has_payload(self):
            if hasattr(self, '_m_has_payload'):
                return self._m_has_payload if hasattr(self, '_m_has_payload') else None

            self._m_has_payload = ((self.control & 32) >> 5)
            return self._m_has_payload if hasattr(self, '_m_has_payload') else None

        @property
        def arq_on(self):
            if hasattr(self, '_m_arq_on'):
                return self._m_arq_on if hasattr(self, '_m_arq_on') else None

            self._m_arq_on = ((self.control & 16) >> 4)
            return self._m_arq_on if hasattr(self, '_m_arq_on') else None

        @property
        def is_authenticated(self):
            if hasattr(self, '_m_is_authenticated'):
                return self._m_is_authenticated if hasattr(self, '_m_is_authenticated') else None

            self._m_is_authenticated = ((self.control & 8) >> 3)
            return self._m_is_authenticated if hasattr(self, '_m_is_authenticated') else None

        @property
        def vc(self):
            if hasattr(self, '_m_vc'):
                return self._m_vc if hasattr(self, '_m_vc') else None

            self._m_vc = ((self.control & 7) >> 0)
            return self._m_vc if hasattr(self, '_m_vc') else None


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Foresail1.Callsign(_io__raw_callsign_ror, self, self._root)


    class ObcHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = Foresail1.UnixTimestamp(self._io, self, self._root)
            self.side = self._io.read_u1()
            self.fdir = self._io.read_u1()
            self.scheduler = self._io.read_u1()
            self.software_revision = self._io.read_u1()
            self.uptime = self._io.read_u4le()
            self.heap_free = self._io.read_u1()
            self.cpu_load = self._io.read_u1()
            self.fs_free_space = self._io.read_u2le()
            self.arbiter_uptime = self._io.read_u2le()
            self.arbiter_age = self._io.read_u2le()
            self.arbiter_bootcount = self._io.read_u2le()
            self.arbiter_temperature = self._io.read_s2le()
            self.side_a_bootcount = self._io.read_u1()
            self.side_a_heartbeat = self._io.read_u1()
            self.side_a_fail_counter = self._io.read_u1()
            self.side_a_fail_reason = self._io.read_u1()
            self.side_b_bootcount = self._io.read_u1()
            self.side_b_heartbeat = self._io.read_u1()
            self.side_b_fail_counter = self._io.read_u1()
            self.side_b_fail_reason = self._io.read_u1()
            self.arbiter_log_1 = self._io.read_u1()
            self.arbiter_log_2 = self._io.read_u1()
            self.arbiter_log_3 = self._io.read_u1()
            self.arbiter_log_4 = self._io.read_u1()






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Greencube(KaitaiStruct):
    """:field header: greencube_raw_frame.csp_header.header
    :field priority: greencube_raw_frame.csp_header.priority
    :field src: greencube_raw_frame.csp_header.src
    :field dest: greencube_raw_frame.csp_header.dest
    :field dest_port: greencube_raw_frame.csp_header.dest_port
    :field src_port: greencube_raw_frame.csp_header.src_port
    :field hmac: greencube_raw_frame.csp_header.hmac
    :field xtea: greencube_raw_frame.csp_header.xtea
    :field rdp: greencube_raw_frame.csp_header.rdp
    :field crc: greencube_raw_frame.csp_header.crc
    :field tlm_id: greencube_raw_frame.tlm_id
    :field unix_time_ms: greencube_raw_frame.tlm_data.unix_time_ms
    :field unix_time: greencube_raw_frame.tlm_data.unix_time
    :field process_time: greencube_raw_frame.tlm_data.process_time
    :field solar_x_voltage: greencube_raw_frame.tlm_data.solar_x_voltage
    :field solar_y_voltage: greencube_raw_frame.tlm_data.solar_y_voltage
    :field solar_x_current: greencube_raw_frame.tlm_data.solar_x_current
    :field solar_y_current: greencube_raw_frame.tlm_data.solar_y_current
    :field eps_bootcause: greencube_raw_frame.tlm_data.eps_bootcause
    :field mppt_x_temperature: greencube_raw_frame.tlm_data.mppt_x_temperature
    :field mppt_y_temperature: greencube_raw_frame.tlm_data.mppt_y_temperature
    :field eps_temperature: greencube_raw_frame.tlm_data.eps_temperature
    :field battery_1_temperature: greencube_raw_frame.tlm_data.battery_1_temperature
    :field battery_2_temperature: greencube_raw_frame.tlm_data.battery_2_temperature
    :field battery_3_temperature: greencube_raw_frame.tlm_data.battery_3_temperature
    :field solar_total_current: greencube_raw_frame.tlm_data.solar_total_current
    :field system_total_current: greencube_raw_frame.tlm_data.system_total_current
    :field battery_voltage: greencube_raw_frame.tlm_data.battery_voltage
    :field eps_outputs: greencube_raw_frame.tlm_data.eps_outputs
    :field radio_pa_temperature: greencube_raw_frame.tlm_data.radio_pa_temperature
    :field radio_tx_count: greencube_raw_frame.tlm_data.radio_tx_count
    :field radio_rx_count: greencube_raw_frame.tlm_data.radio_rx_count
    :field radio_last_rssi: greencube_raw_frame.tlm_data.radio_last_rssi
    :field obc_boot_count: greencube_raw_frame.tlm_data.obc_boot_count
    :field radio_boot_count: greencube_raw_frame.tlm_data.radio_boot_count
    :field eps_boot_count: greencube_raw_frame.tlm_data.eps_boot_count
    :field payload_rx_count: greencube_raw_frame.tlm_data.payload_rx_count
    :field payload_tx_count: greencube_raw_frame.tlm_data.payload_tx_count
    :field software_status: greencube_raw_frame.tlm_data.software_status
    :field heater_status: greencube_raw_frame.tlm_data.heater_status
    :field obc_temperature: greencube_raw_frame.tlm_data.obc_temperature
    :field gyroscope_x: greencube_raw_frame.tlm_data.gyroscope_x
    :field gyroscope_y: greencube_raw_frame.tlm_data.gyroscope_y
    :field gyroscope_z: greencube_raw_frame.tlm_data.gyroscope_z
    :field magnetometer_x: greencube_raw_frame.tlm_data.magnetometer_x
    :field magnetometer_y: greencube_raw_frame.tlm_data.magnetometer_y
    :field magnetometer_z: greencube_raw_frame.tlm_data.magnetometer_z
    :field solarpanel_plus_x_temperature: greencube_raw_frame.tlm_data.solarpanel_plus_x_temperature
    :field solarpanel_plus_y_temperature: greencube_raw_frame.tlm_data.solarpanel_plus_y_temperature
    :field solarpanel_minus_x_temperature: greencube_raw_frame.tlm_data.solarpanel_minus_x_temperature
    :field solarpanel_minus_y_temperature: greencube_raw_frame.tlm_data.solarpanel_minus_y_temperature
    :field coarse_sun_sensor_plus_x: greencube_raw_frame.tlm_data.coarse_sun_sensor_plus_x
    :field coarse_sun_sensor_plus_y: greencube_raw_frame.tlm_data.coarse_sun_sensor_plus_y
    :field coarse_sun_sensor_minus_x: greencube_raw_frame.tlm_data.coarse_sun_sensor_minus_x
    :field coarse_sun_sensor_minus_y: greencube_raw_frame.tlm_data.coarse_sun_sensor_minus_y
    :field gps_status_flag: greencube_raw_frame.tlm_data.gps_status_flag
    :field gps_fix_time: greencube_raw_frame.tlm_data.gps_fix_time
    :field gps_latitude: greencube_raw_frame.tlm_data.gps_latitude
    :field gps_longitude: greencube_raw_frame.tlm_data.gps_longitude
    :field gps_altitude: greencube_raw_frame.tlm_data.gps_altitude
    :field sband_tx_count: greencube_raw_frame.tlm_data.sband_tx_count
    :field antenna_status_flag: greencube_raw_frame.tlm_data.antenna_status_flag
    :field acs_state_flag: greencube_raw_frame.tlm_data.acs_state_flag
    :field acs_parameters_flag: greencube_raw_frame.tlm_data.acs_parameters_flag
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        _on = self._io.size()
        if _on == 101:
            self.greencube_raw_frame = Greencube.GreencubeTlmFrame(self._io, self, self._root)

    class GreencubeTlmFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.csp_header = Greencube.CspHeader(self._io, self, self._root)
            self.tlm_id = self._io.read_u2be()
            if not  ((self.tlm_id == 13840) or (self.tlm_id == 13841) or (self.tlm_id == 13842) or (self.tlm_id == 30234)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.tlm_id, self._io, u"/types/greencube_tlm_frame/seq/1")
            self.tlm_data = Greencube.TlmData(self._io, self, self._root)


    class CspHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = self._io.read_u4be()

        @property
        def src(self):
            if hasattr(self, '_m_src'):
                return self._m_src if hasattr(self, '_m_src') else None

            self._m_src = ((self.header >> 25) & 31)
            return self._m_src if hasattr(self, '_m_src') else None

        @property
        def dest(self):
            if hasattr(self, '_m_dest'):
                return self._m_dest if hasattr(self, '_m_dest') else None

            self._m_dest = ((self.header >> 20) & 31)
            return self._m_dest if hasattr(self, '_m_dest') else None

        @property
        def rdp(self):
            if hasattr(self, '_m_rdp'):
                return self._m_rdp if hasattr(self, '_m_rdp') else None

            self._m_rdp = ((self.header >> 1) & 1)
            return self._m_rdp if hasattr(self, '_m_rdp') else None

        @property
        def src_port(self):
            if hasattr(self, '_m_src_port'):
                return self._m_src_port if hasattr(self, '_m_src_port') else None

            self._m_src_port = ((self.header >> 8) & 63)
            return self._m_src_port if hasattr(self, '_m_src_port') else None

        @property
        def priority(self):
            if hasattr(self, '_m_priority'):
                return self._m_priority if hasattr(self, '_m_priority') else None

            self._m_priority = ((self.header >> 30) & 3)
            return self._m_priority if hasattr(self, '_m_priority') else None

        @property
        def xtea(self):
            if hasattr(self, '_m_xtea'):
                return self._m_xtea if hasattr(self, '_m_xtea') else None

            self._m_xtea = ((self.header >> 2) & 1)
            return self._m_xtea if hasattr(self, '_m_xtea') else None

        @property
        def hmac(self):
            if hasattr(self, '_m_hmac'):
                return self._m_hmac if hasattr(self, '_m_hmac') else None

            self._m_hmac = ((self.header >> 3) & 1)
            return self._m_hmac if hasattr(self, '_m_hmac') else None

        @property
        def crc(self):
            if hasattr(self, '_m_crc'):
                return self._m_crc if hasattr(self, '_m_crc') else None

            self._m_crc = (self.header & 1)
            return self._m_crc if hasattr(self, '_m_crc') else None

        @property
        def dest_port(self):
            if hasattr(self, '_m_dest_port'):
                return self._m_dest_port if hasattr(self, '_m_dest_port') else None

            self._m_dest_port = ((self.header >> 14) & 63)
            return self._m_dest_port if hasattr(self, '_m_dest_port') else None


    class TlmData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unix_time_ms = self._io.read_u2be()
            self.unix_time = self._io.read_u4be()
            self.process_time = self._io.read_u2be()
            self.solar_x_voltage = self._io.read_u2be()
            self.solar_y_voltage = self._io.read_u2be()
            self.solar_x_current = self._io.read_u2be()
            self.solar_y_current = self._io.read_u2be()
            self.eps_bootcause = self._io.read_u1()
            self.mppt_x_temperature = self._io.read_s1()
            self.mppt_y_temperature = self._io.read_s1()
            self.eps_temperature = self._io.read_s1()
            self.battery_1_temperature = self._io.read_s1()
            self.battery_2_temperature = self._io.read_s1()
            self.battery_3_temperature = self._io.read_s1()
            self.solar_total_current = self._io.read_u2be()
            self.system_total_current = self._io.read_u2be()
            self.battery_voltage = self._io.read_u2be()
            self.eps_outputs = self._io.read_u1()
            self.radio_pa_temperature = self._io.read_s1()
            self.radio_tx_count = self._io.read_u2be()
            self.radio_rx_count = self._io.read_u2be()
            self.radio_last_rssi = self._io.read_s2be()
            self.obc_boot_count = self._io.read_u2be()
            self.radio_boot_count = self._io.read_u2be()
            self.eps_boot_count = self._io.read_u2be()
            self.payload_rx_count = self._io.read_u1()
            self.payload_tx_count = self._io.read_u1()
            self.software_status = self._io.read_u1()
            self.heater_status = self._io.read_u1()
            self.obc_temperature = self._io.read_u1()
            self.gyroscope_x = self._io.read_s2be()
            self.gyroscope_y = self._io.read_s2be()
            self.gyroscope_z = self._io.read_s2be()
            self.magnetometer_x = self._io.read_s2be()
            self.magnetometer_y = self._io.read_s2be()
            self.magnetometer_z = self._io.read_s2be()
            self.solarpanel_plus_x_temperature = self._io.read_s1()
            self.solarpanel_plus_y_temperature = self._io.read_s1()
            self.solarpanel_minus_x_temperature = self._io.read_s1()
            self.solarpanel_minus_y_temperature = self._io.read_s1()
            self.coarse_sun_sensor_plus_x = self._io.read_u2be()
            self.coarse_sun_sensor_plus_y = self._io.read_u2be()
            self.coarse_sun_sensor_minus_x = self._io.read_u2be()
            self.coarse_sun_sensor_minus_y = self._io.read_u2be()
            self.gps_status_flag = self._io.read_u1()
            self.gps_fix_time = self._io.read_u4be()
            self.gps_latitude = self._io.read_u4be()
            self.gps_longitude = self._io.read_u4be()
            self.gps_altitude = self._io.read_u4be()
            self.sband_tx_count = self._io.read_u2be()
            self.antenna_status_flag = self._io.read_u2be()
            self.acs_state_flag = self._io.read_u1()
            self.acs_parameters_flag = self._io.read_u1()






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Qarman(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field ssid_mask: ax25_frame.ax25_header.dest_ssid_raw.ssid_mask
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field ssid_mask: ax25_frame.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field battery_voltage: ax25_frame.payload.data_payload.mode.eps_payload.battery_voltage
    :field temperature_obc: ax25_frame.payload.data_payload.mode.eps_payload.temperature_obc
    :field battery_current: ax25_frame.payload.data_payload.mode.eps_payload.battery_current
    :field reg_bus_3v3_current: ax25_frame.payload.data_payload.mode.eps_payload.reg_bus_3v3_current
    :field reg_bus_5v0_current: ax25_frame.payload.data_payload.mode.eps_payload.reg_bus_5v0_current
    :field temperature_uhf: ax25_frame.payload.data_payload.mode.eps_payload.temperature_uhf
    :field obc_mode: ax25_frame.payload.data_payload.mode.eps_payload.obc_mode
    :field reason_for_mode_change: ax25_frame.payload.data_payload.mode.eps_payload.reason_for_mode_change
    :field obc_uptime: ax25_frame.payload.data_payload.mode.eps_payload.obc_uptime
    :field obc_boot_counter: ax25_frame.payload.data_payload.mode.eps_payload.obc_boot_counter
    :field obc_packet_counter: ax25_frame.payload.data_payload.mode.eps_payload.obc_packet_counter
    :field obc_tc_frm_received: ax25_frame.payload.data_payload.mode.eps_payload.obc_tc_frm_received
    :field obc_tc_valid: ax25_frame.payload.data_payload.mode.eps_payload.obc_tc_valid
    :field systems_on: ax25_frame.payload.data_payload.mode.eps_payload.systems_on
    :field deployable_status: ax25_frame.payload.data_payload.mode.eps_payload.deployable_status
    :field solar_panel_current_posxi: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_posxi
    :field solar_panel_current_posxo: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_posxo
    :field solar_panel_current_negyi: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_negyi
    :field solar_panel_current_negyo: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_negyo
    :field solar_panel_current_negxi: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_negxi
    :field solar_panel_current_negxo: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_negxo
    :field solar_panel_current_posyi: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_posyi
    :field solar_panel_current_posyo: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_posyo
    :field solar_panel_voltage_posx: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_voltage_posx
    :field solar_panel_voltage_negy: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_voltage_negy
    :field solar_panel_voltage_negx: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_voltage_negx
    :field solar_panel_voltage_posy: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_voltage_posy
    :field battery_voltage: ax25_frame.payload.data_payload.mode.eps_payload.battery_voltage
    :field temperature_obc: ax25_frame.payload.data_payload.mode.eps_payload.temperature_obc
    :field battery_current: ax25_frame.payload.data_payload.mode.eps_payload.battery_current
    :field reg_bus_3v3_current: ax25_frame.payload.data_payload.mode.eps_payload.reg_bus_3v3_current
    :field reg_bus_5v0_current: ax25_frame.payload.data_payload.mode.eps_payload.reg_bus_5v0_current
    :field temperature_uhf: ax25_frame.payload.data_payload.mode.eps_payload.temperature_uhf
    :field obc_mode: ax25_frame.payload.data_payload.mode.eps_payload.obc_mode
    :field reason_for_mode_change: ax25_frame.payload.data_payload.mode.eps_payload.reason_for_mode_change
    :field obc_uptime: ax25_frame.payload.data_payload.mode.eps_payload.obc_uptime
    :field obc_boot_counter: ax25_frame.payload.data_payload.mode.eps_payload.obc_boot_counter
    :field obc_packet_counter: ax25_frame.payload.data_payload.mode.eps_payload.obc_packet_counter
    :field obc_tc_frm_received: ax25_frame.payload.data_payload.mode.eps_payload.obc_tc_frm_received
    :field obc_tc_valid: ax25_frame.payload.data_payload.mode.eps_payload.obc_tc_valid
    :field systems_on: ax25_frame.payload.data_payload.mode.eps_payload.systems_on
    :field deployable_status: ax25_frame.payload.data_payload.mode.eps_payload.deployable_status
    :field solar_panel_current_posxi: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_posxi
    :field solar_panel_current_posxo: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_posxo
    :field solar_panel_current_negyi: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_negyi
    :field solar_panel_current_negyo: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_negyo
    :field solar_panel_current_negxi: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_negxi
    :field solar_panel_current_negxo: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_negxo
    :field solar_panel_current_posyi: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_posyi
    :field solar_panel_current_posyo: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_current_posyo
    :field solar_panel_voltage_posx: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_voltage_posx
    :field solar_panel_voltage_negy: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_voltage_negy
    :field solar_panel_voltage_negx: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_voltage_negx
    :field solar_panel_voltage_posy: ax25_frame.payload.data_payload.mode.eps_payload.solar_panel_voltage_posy
    :field adcs_state: ax25_frame.payload.data_payload.mode.adcs_payload.adcs_state
    :field attitude_estimation_mode: ax25_frame.payload.data_payload.mode.adcs_payload.attitude_estimation_mode
    :field control_mode: ax25_frame.payload.data_payload.mode.adcs_payload.control_mode
    :field cube_control_3v3_current: ax25_frame.payload.data_payload.mode.adcs_payload.cube_control_3v3_current
    :field cube_control_5v_current: ax25_frame.payload.data_payload.mode.adcs_payload.cube_control_5v_current
    :field cube_control_vbat_current: ax25_frame.payload.data_payload.mode.adcs_payload.cube_control_vbat_current
    :field magnetorquer_current: ax25_frame.payload.data_payload.mode.adcs_payload.magnetorquer_current
    :field momentum_wheel_current: ax25_frame.payload.data_payload.mode.adcs_payload.momentum_wheel_current
    :field magnetic_field_x: ax25_frame.payload.data_payload.mode.adcs_payload.magnetic_field_x
    :field magnetic_field_y: ax25_frame.payload.data_payload.mode.adcs_payload.magnetic_field_y
    :field magnetic_field_z: ax25_frame.payload.data_payload.mode.adcs_payload.magnetic_field_z
    :field y_angular_rate: ax25_frame.payload.data_payload.mode.adcs_payload.y_angular_rate
    :field y_wheel_speed: ax25_frame.payload.data_payload.mode.adcs_payload.y_wheel_speed
    :field estimated_roll_angle: ax25_frame.payload.data_payload.mode.adcs_payload.estimated_roll_angle
    :field estimated_pitch_angle: ax25_frame.payload.data_payload.mode.adcs_payload.estimated_pitch_angle
    :field estimated_yaw_angle: ax25_frame.payload.data_payload.mode.adcs_payload.estimated_yaw_angle
    :field estimated_x_angular_rate: ax25_frame.payload.data_payload.mode.adcs_payload.estimated_x_angular_rate
    :field estimated_y_angular_rate: ax25_frame.payload.data_payload.mode.adcs_payload.estimated_y_angular_rate
    :field estimated_z_angular_rate: ax25_frame.payload.data_payload.mode.adcs_payload.estimated_z_angular_rate
    :field temperature_adcs_arm_cpu: ax25_frame.payload.data_payload.mode.adcs_payload.temperature_adcs_arm_cpu
    :field padding: ax25_frame.payload.data_payload.mode.adcs_payload.padding
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Qarman.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Qarman.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Qarman.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Qarman.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Qarman.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Qarman.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Qarman.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Qarman.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Qarman.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Qarman.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Qarman.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Qarman.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class AdcsPayloadT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.adcs_state = self._io.read_bits_int_be(2)
            self.attitude_estimation_mode = self._io.read_bits_int_be(3)
            self.control_mode = self._io.read_bits_int_be(3)
            self.cube_control_3v3_current = self._io.read_bits_int_be(16)
            self.cube_control_5v_current = self._io.read_bits_int_be(16)
            self.cube_control_vbat_current = self._io.read_bits_int_be(16)
            self.magnetorquer_current = self._io.read_bits_int_be(16)
            self.momentum_wheel_current = self._io.read_bits_int_be(16)
            self.magnetic_field_x = self._io.read_bits_int_be(16)
            self.magnetic_field_y = self._io.read_bits_int_be(16)
            self.magnetic_field_z = self._io.read_bits_int_be(16)
            self.y_angular_rate = self._io.read_bits_int_be(16)
            self.y_wheel_speed = self._io.read_bits_int_be(16)
            self.estimated_roll_angle = self._io.read_bits_int_be(16)
            self.estimated_pitch_angle = self._io.read_bits_int_be(16)
            self.estimated_yaw_angle = self._io.read_bits_int_be(16)
            self.estimated_x_angular_rate = self._io.read_bits_int_be(16)
            self.estimated_y_angular_rate = self._io.read_bits_int_be(16)
            self.estimated_z_angular_rate = self._io.read_bits_int_be(16)
            self.temperature_adcs_arm_cpu = self._io.read_bits_int_be(16)
            self.padding = self._io.read_bits_int_be(4)


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            _on = self._parent.ax25_header.src_callsign_raw.callsign_ror.callsign
            if _on == u"ON05BE":
                self.data_payload = Qarman.QarmanPayloadT(self._io, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class EpsPayloadT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.battery_voltage = self._io.read_bits_int_be(12)
            self.temperature_obc = self._io.read_bits_int_be(12)
            self.battery_current = self._io.read_bits_int_be(10)
            self.reg_bus_3v3_current = self._io.read_bits_int_be(10)
            self.reg_bus_5v0_current = self._io.read_bits_int_be(10)
            self.temperature_uhf = self._io.read_bits_int_be(16)
            self.obc_mode = self._io.read_bits_int_be(4)
            self.reason_for_mode_change = self._io.read_bits_int_be(3)
            self.obc_uptime = self._io.read_bits_int_be(32)
            self.obc_boot_counter = self._io.read_bits_int_be(8)
            self.obc_packet_counter = self._io.read_bits_int_be(16)
            self.obc_tc_frm_received = self._io.read_bits_int_be(8)
            self.obc_tc_valid = self._io.read_bits_int_be(8)
            self.systems_on = self._io.read_bits_int_be(26)
            self.deployable_status = self._io.read_bits_int_be(13)
            self.solar_panel_current_posxi = self._io.read_bits_int_be(10)
            self.solar_panel_current_posxo = self._io.read_bits_int_be(10)
            self.solar_panel_current_negyi = self._io.read_bits_int_be(10)
            self.solar_panel_current_negyo = self._io.read_bits_int_be(10)
            self.solar_panel_current_negxi = self._io.read_bits_int_be(10)
            self.solar_panel_current_negxo = self._io.read_bits_int_be(10)
            self.solar_panel_current_posyi = self._io.read_bits_int_be(10)
            self.solar_panel_current_posyo = self._io.read_bits_int_be(10)
            self.solar_panel_voltage_posx = self._io.read_bits_int_be(10)
            self.solar_panel_voltage_negy = self._io.read_bits_int_be(10)
            self.solar_panel_voltage_negx = self._io.read_bits_int_be(10)
            self.solar_panel_voltage_posy = self._io.read_bits_int_be(10)


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class LpBeaconT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.eps_payload = Qarman.EpsPayloadT(self._io, self, self._root)


    class QarmanPayloadT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self._root.framelength
            if _on == 55:
                self.mode = Qarman.LpBeaconT(self._io, self, self._root)
            elif _on == 90:
                self.mode = Qarman.FullBeaconT(self._io, self, self._root)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Qarman.Callsign(_io__raw_callsign_ror, self, self._root)


    class FullBeaconT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.eps_payload = Qarman.EpsPayloadT(self._io, self, self._root)
            self.adcs_payload = Qarman.AdcsPayloadT(self._io, self, self._root)


    @property
    def framelength(self):
        if hasattr(self, '_m_framelength'):
            return self._m_framelength if hasattr(self, '_m_framelength') else None

        self._m_framelength = self._io.size()
        return self._m_framelength if hasattr(self, '_m_framelength') else None





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Selfiesat(KaitaiStruct):
    """:field crc: csp_header.crc
    :field rdp: csp_header.rdp
    :field xtea: csp_header.xtea
    :field hmac: csp_header.hmac
    :field reserved: csp_header.reserved
    :field src_port: csp_header.src_port
    :field dst_port: csp_header.dst_port
    :field destination: csp_header.destination
    :field source: csp_header.source
    :field priority: csp_header.priority
    :field packet_length: csp_data.packet_length
    :field alarm_mask: csp_data.csp_payload.alarm_mask
    :field eps_counter_boot: csp_data.csp_payload.eps_counter_boot
    :field eps_vbatt: csp_data.csp_payload.eps_vbatt
    :field eps_outputmask: csp_data.csp_payload.eps_outputmask
    :field id: csp_data.csp_payload.id
    :field fsm_states: csp_data.csp_payload.fsm_states
    :field callsign: csp_data.csp_payload.callsign
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.csp_header = Selfiesat.CspHeaderT(self._io, self, self._root)
        self.csp_data = Selfiesat.CspDataT(self._io, self, self._root)

    class CspHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.csp_flags = [None] * (4)
            for i in range(4):
                self.csp_flags[i] = self._io.read_u1()


        @property
        def source(self):
            if hasattr(self, '_m_source'):
                return self._m_source if hasattr(self, '_m_source') else None

            self._m_source = ((self.csp_flags[3] >> 1) & 31)
            return self._m_source if hasattr(self, '_m_source') else None

        @property
        def rdp(self):
            if hasattr(self, '_m_rdp'):
                return self._m_rdp if hasattr(self, '_m_rdp') else None

            self._m_rdp = ((self.csp_flags[0] >> 1) & 1)
            return self._m_rdp if hasattr(self, '_m_rdp') else None

        @property
        def src_port(self):
            if hasattr(self, '_m_src_port'):
                return self._m_src_port if hasattr(self, '_m_src_port') else None

            self._m_src_port = (self.csp_flags[1] & 63)
            return self._m_src_port if hasattr(self, '_m_src_port') else None

        @property
        def destination(self):
            if hasattr(self, '_m_destination'):
                return self._m_destination if hasattr(self, '_m_destination') else None

            self._m_destination = (((self.csp_flags[2] | (self.csp_flags[3] << 8)) >> 4) & 31)
            return self._m_destination if hasattr(self, '_m_destination') else None

        @property
        def dst_port(self):
            if hasattr(self, '_m_dst_port'):
                return self._m_dst_port if hasattr(self, '_m_dst_port') else None

            self._m_dst_port = (((self.csp_flags[1] | (self.csp_flags[2] << 8)) >> 6) & 63)
            return self._m_dst_port if hasattr(self, '_m_dst_port') else None

        @property
        def priority(self):
            if hasattr(self, '_m_priority'):
                return self._m_priority if hasattr(self, '_m_priority') else None

            self._m_priority = (self.csp_flags[3] >> 6)
            return self._m_priority if hasattr(self, '_m_priority') else None

        @property
        def reserved(self):
            if hasattr(self, '_m_reserved'):
                return self._m_reserved if hasattr(self, '_m_reserved') else None

            self._m_reserved = (self.csp_flags[0] >> 4)
            return self._m_reserved if hasattr(self, '_m_reserved') else None

        @property
        def xtea(self):
            if hasattr(self, '_m_xtea'):
                return self._m_xtea if hasattr(self, '_m_xtea') else None

            self._m_xtea = ((self.csp_flags[0] >> 2) & 1)
            return self._m_xtea if hasattr(self, '_m_xtea') else None

        @property
        def hmac(self):
            if hasattr(self, '_m_hmac'):
                return self._m_hmac if hasattr(self, '_m_hmac') else None

            self._m_hmac = ((self.csp_flags[0] >> 3) & 1)
            return self._m_hmac if hasattr(self, '_m_hmac') else None

        @property
        def crc(self):
            if hasattr(self, '_m_crc'):
                return self._m_crc if hasattr(self, '_m_crc') else None

            self._m_crc = (self.csp_flags[0] & 1)
            return self._m_crc if hasattr(self, '_m_crc') else None


    class CspDataT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.packet_length = self._io.read_u2be()
            if not  ((self.packet_length == 224)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.packet_length, self._io, u"/types/csp_data_t/seq/0")
            self.pad_0 = self._io.read_bytes(8)
            self._raw_csp_payload = self._io.read_bytes_full()
            _io__raw_csp_payload = KaitaiStream(BytesIO(self._raw_csp_payload))
            self.csp_payload = Selfiesat.SelfiesatTelemetryT(_io__raw_csp_payload, self, self._root)


    class SelfiesatTelemetryT(KaitaiStruct):
        """struct bcn_normal {
          unsigned int alarm_mask;            // Alarms from housekeeping
          unsigned short eps_counter_boot;    // EPS boot count
          unsigned short eps_vbatt;           // Voltage of EPS battery
          unsigned char eps_outputmask;       // Whether channels are on or off
          unsigned char id;                   // Identifier for beacons
          unsigned char fsm_states;           // obc_main's internal fsm states
        };
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.alarm_mask = self._io.read_u4be()
            self.eps_counter_boot = self._io.read_u2be()
            self.eps_vbatt = self._io.read_u2be()
            self.eps_outputmask = self._io.read_u1()
            self.id = self._io.read_u1()
            self.fsm_states = self._io.read_u1()
            self.pad_1 = self._io.read_bytes(1)
            self.callsign = (KaitaiStream.bytes_terminate(self._io.read_bytes(9), 0, False)).decode(u"ASCII")
            if not  ((self.callsign == u"SelfieSat")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/selfiesat_telemetry_t/seq/7")






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Uwe4(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field beacon_header_flags1: ax25_frame.payload.ax25_info.beacon_header.beacon_header_flags1
    :field beacon_header_flags2: ax25_frame.payload.ax25_info.beacon_header.beacon_header_flags2
    :field beacon_header_packet_id: ax25_frame.payload.ax25_info.beacon_header.beacon_header_packet_id
    :field beacon_header_fm_system_id: ax25_frame.payload.ax25_info.beacon_header.beacon_header_fm_system_id
    :field beacon_header_fm_subsystem_id: ax25_frame.payload.ax25_info.beacon_header.beacon_header_fm_subsystem_id
    :field beacon_header_to_system_id: ax25_frame.payload.ax25_info.beacon_header.beacon_header_to_system_id
    :field beacon_header_to_subsystem_id: ax25_frame.payload.ax25_info.beacon_header.beacon_header_to_subsystem_id
    :field beacon_header_api: ax25_frame.payload.ax25_info.beacon_header.beacon_header_api
    :field beacon_header_payload_size: ax25_frame.payload.ax25_info.beacon_header.beacon_header_payload_size
    :field beacon_payload_command: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_command
    :field beacon_payload_var_id: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_var_id
    :field beacon_payload_typeandlength: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_typeandlength
    :field beacon_payload_timestamp: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_timestamp
    :field beacon_payload_beacon_rate: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_beacon_rate
    :field beacon_payload_vals_out_of_range: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_vals_out_of_range
    :field beacon_payload_uptime: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_uptime
    :field beacon_payload_subsystem_status_bitmap: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_subsystem_status.beacon_payload_subsystem_status_bitmap
    :field beacon_payload_batt_a_temp: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_batt_a_temp
    :field beacon_payload_batt_a_state_of_charge: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_batt_a_state_of_charge
    :field beacon_payload_batt_b_temp: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_batt_b_temp
    :field beacon_payload_batt_b_state_of_charge: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_batt_b_state_of_charge
    :field beacon_payload_batt_a_current: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_batt_a_current
    :field beacon_payload_batt_a_voltage: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_batt_a_voltage
    :field beacon_payload_batt_b_current: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_batt_b_current
    :field beacon_payload_batt_b_voltage: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_batt_b_voltage
    :field beacon_payload_power_consumption: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_power_consumption
    :field beacon_payload_obc_temp: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_obc_temp
    :field beacon_payload_panel_pos_x_temp: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_panel_pos_x_temp
    :field beacon_payload_panel_neg_x_temp: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_panel_neg_x_temp
    :field beacon_payload_panel_pos_y_temp: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_panel_pos_y_temp
    :field beacon_payload_panel_neg_y_temp: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_panel_neg_y_temp
    :field beacon_payload_panel_pos_z_temp: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_panel_pos_z_temp
    :field beacon_payload_panel_neg_z_temp: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_panel_neg_z_temp
    :field beacon_payload_freq: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_freq
    :field beacon_payload_crc: ax25_frame.payload.ax25_info.beacon_payload.beacon_payload_crc
    :field rf_message: ax25_frame.payload.ax25_info.beacon_payload.message
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Uwe4.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Uwe4.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Uwe4.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Uwe4.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Uwe4.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Uwe4.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Uwe4.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Uwe4.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Uwe4.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Uwe4.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Uwe4.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Uwe4.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class HskpPayload(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_payload_command = self._io.read_u1()
            self.beacon_payload_var_id = self._io.read_u2le()
            self.beacon_payload_typeandlength = self._io.read_u2le()
            self.beacon_payload_timestamp_raw = [None] * (6)
            for i in range(6):
                self.beacon_payload_timestamp_raw[i] = self._io.read_u1()

            self.beacon_payload_beacon_rate = self._io.read_u4le()
            self.beacon_payload_vals_out_of_range = self._io.read_u2le()
            self.beacon_payload_uptime = self._io.read_u4le()
            self.beacon_payload_subsystem_status = Uwe4.Bitmap16SubsystemStatus(self._io, self, self._root)
            self.beacon_payload_batt_a_temp = self._io.read_s1()
            self.beacon_payload_batt_a_state_of_charge = self._io.read_s1()
            self.beacon_payload_batt_b_temp = self._io.read_s1()
            self.beacon_payload_batt_b_state_of_charge = self._io.read_s1()
            self.beacon_payload_batt_a_current = self._io.read_s2le()
            self.beacon_payload_batt_a_voltage = self._io.read_s2le()
            self.beacon_payload_batt_b_current = self._io.read_s2le()
            self.beacon_payload_batt_b_voltage = self._io.read_s2le()
            self.beacon_payload_power_consumption = self._io.read_s2le()
            self.beacon_payload_obc_temp = self._io.read_s1()
            self.beacon_payload_panel_pos_x_temp = self._io.read_s1()
            self.beacon_payload_panel_neg_x_temp = self._io.read_s1()
            self.beacon_payload_panel_pos_y_temp = self._io.read_s1()
            self.beacon_payload_panel_neg_y_temp = self._io.read_s1()
            self.beacon_payload_panel_pos_z_temp = self._io.read_s1()
            self.beacon_payload_panel_neg_z_temp = self._io.read_s1()
            self.beacon_payload_freq = self._io.read_u2le()
            self.beacon_payload_crc = self._io.read_u2le()

        @property
        def beacon_payload_timestamp(self):
            if hasattr(self, '_m_beacon_payload_timestamp'):
                return self._m_beacon_payload_timestamp if hasattr(self, '_m_beacon_payload_timestamp') else None

            self._m_beacon_payload_timestamp = (((((self.beacon_payload_timestamp_raw[0] + (self.beacon_payload_timestamp_raw[1] << 8)) + (self.beacon_payload_timestamp_raw[2] << 16)) + (self.beacon_payload_timestamp_raw[3] << 24)) + (self.beacon_payload_timestamp_raw[4] << 32)) + (self.beacon_payload_timestamp_raw[5] << 48))
            return self._m_beacon_payload_timestamp if hasattr(self, '_m_beacon_payload_timestamp') else None


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Uwe4.Beacon(_io__raw_ax25_info, self, self._root)


    class Bitmap16SubsystemStatus(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_payload_subsystem_status_bitmap = self._io.read_u2le()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"utf-8")


    class RfMessage(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.offset_0 = [None] * (6)
            for i in range(6):
                self.offset_0[i] = self._io.read_u1()

            self.message = (self._io.read_bytes((self._parent.beacon_header.beacon_header_payload_size - 6))).decode(u"utf-8")
            self.rf_message_crc = self._io.read_u2le()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class Beacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.is_valid_source
            if _on == True:
                self.beacon_header = Uwe4.BeaconHeader(self._io, self, self._root)
            if self.is_valid_payload:
                _on = self.beacon_header.beacon_header_api
                if _on == 14:
                    self.beacon_payload = Uwe4.HskpPayload(self._io, self, self._root)
                elif _on == 103:
                    self.beacon_payload = Uwe4.RfMessage(self._io, self, self._root)


        @property
        def is_valid_source(self):
            """This is work in progress as it never returns `true` without the
            `(1 == 1)` statement. It DOES NOT check the source for now!
            """
            if hasattr(self, '_m_is_valid_source'):
                return self._m_is_valid_source if hasattr(self, '_m_is_valid_source') else None

            self._m_is_valid_source =  ((1 == 1) or (self._root.ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign == u"DP0UWH")) 
            return self._m_is_valid_source if hasattr(self, '_m_is_valid_source') else None

        @property
        def is_valid_payload(self):
            if hasattr(self, '_m_is_valid_payload'):
                return self._m_is_valid_payload if hasattr(self, '_m_is_valid_payload') else None

            self._m_is_valid_payload =  (( ((self.beacon_header.beacon_header_fm_system_id == 2) and (self.beacon_header.beacon_header_fm_subsystem_id == 1) and (self.beacon_header.beacon_header_to_system_id == 1) and (self.beacon_header.beacon_header_to_subsystem_id == 0) and (self.beacon_header.beacon_header_payload_size == 46) and (self.beacon_header.beacon_header_api == 14)) ) or ( ((self.beacon_header.beacon_header_fm_system_id == 2) and (self.beacon_header.beacon_header_fm_subsystem_id == 1) and (self.beacon_header.beacon_header_to_system_id == 1) and (self.beacon_header.beacon_header_to_subsystem_id == 0) and (self.beacon_header.beacon_header_api == 103)) )) 
            return self._m_is_valid_payload if hasattr(self, '_m_is_valid_payload') else None


    class BeaconHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_header_flags1 = self._io.read_u1()
            self.beacon_header_flags2 = self._io.read_u1()
            self.beacon_header_packet_id = self._io.read_u2le()
            self.beacon_header_fm_system_id = self._io.read_u1()
            self.beacon_header_fm_subsystem_id = self._io.read_u1()
            self.beacon_header_to_system_id = self._io.read_u1()
            self.beacon_header_to_subsystem_id = self._io.read_u1()
            self.beacon_header_api = self._io.read_u1()
            self.beacon_header_payload_size = self._io.read_u1()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Uwe4.Callsign(_io__raw_callsign_ror, self, self._root)






"""
decode single (binary) frames from file or commandline
and multiple frames from a CSV file
"""

from __future__ import print_function

import argparse
import binascii
import json
import re
import sys
from datetime import datetime

from kaitaistruct import KaitaiStructError

from satnogsdecoders import __version__ as decoders_version
from satnogsdecoders import decode_frame


class CliParameters(object):  # pylint: disable=R0903
    """
    Commandline parameters
    """
    def __init__(self, fname=None, hframe=None, fformat=None, tmstamp=None):
        self.fname = fname
        self.hframe = hframe
        self.fformat = fformat
        self.tmstamp = tmstamp


def main():
    """
    main entry point
    """

    parser = argparse.ArgumentParser(
        description='Decode single or multiple frames using a Kaitai'
        'Struct decoder into JSON objects.')
    parser.add_argument('decoder_name',
                        type=str,
                        help='Name of the decoder (e.g. Eshail2).')
    parser.add_argument('--filename',
                        type=str,
                        help='Filename containing the frame[s].')
    parser.add_argument('--hex_frame',
                        type=str,
                        help='Hexlified frame provided as argument like '
                        '"001122AABBCCDDEEFF".')
    parser.add_argument('--format',
                        type=str,
                        help="Input file format, 'bin' or 'csv'.")
    parser.add_argument(
        '--tmstamp',
        type=str,
        help='For single hex or bin frame: timestamp of the current frame.')
    parser.add_argument('-v',
                        action='store_true',
                        help='Enables verbose output mode.')
    args = parser.parse_args()
    parms = CliParameters(args.filename, args.hex_frame, args.format,
                          args.tmstamp)
    print(decode_multiple(args.decoder_name, parms, args.v))


def create_point(fields, timestamp, decoder, version):
    """
    Create a decoded data point in JSON format

    :returns: a JSON formatted time series data point
    """
    point = {
        'time': timestamp,
        'measurement': '',
        'tags': {
            'satellite': '',
            'decoder': decoder,
            'station': '',
            'observer': '',
            'source': '',
            'version': version
        },
        'fields': fields
    }

    return point


def check_valid_format(params):
    """
    Check if frame format in file is valid
    """
    if params.fformat not in ('bin', 'csv'):
        if params.hframe is None:
            print(
                f"Wrong input file format \'{params.fformat}\'!"
                f"Must be 'bin' or 'csv'!",
                file=sys.stderr)
            sys.exit(2)
    return True


def parse_csv_file(dname, params, json_obj, verbose):
    """
    Parser for CSV files
    """

    if params.fformat == 'csv':
        with open(params.fname, encoding='utf-8') as fil:
            lines = [line.rstrip('\n') for line in fil]
        for each_line in lines:
            try:
                # substitute newlines, " and carriage-return
                each_line = re.sub('["\r\n]', '', each_line)
                bindata = binascii.unhexlify(each_line[20:])
                if verbose == 1:
                    print(
                        f'Decoding frame (l={len(bindata)}):'
                        f'{each_line[20:]}\n',
                        file=sys.stderr)
                timestamp = each_line[:19]
                json_obj.append(
                    create_point(decode_frame.decode_frame(dname, bindata),
                                 timestamp, dname, decoders_version))
            # pylint: disable=broad-except
            except (Exception, KaitaiStructError) as error:
                if verbose is True:
                    print('^~~~ Invalid frame!', file=sys.stderr)
                    print(error, file=sys.stderr)
    return json_obj


def parse_bin_file(dname, params, json_obj):
    """
    Parser for binary files
    """
    if params.fformat == 'bin':
        with open(params.fname, 'rb') as file_t:
            bindata = bytearray(file_t.read())
            if params.tmstamp is not None:
                timestamp = params.tmstamp
            else:
                timestamp = datetime.strftime(datetime.now(),
                                              '%Y-%m-%d %H:%M:%S')
                json_obj.append(
                    create_point(decode_frame.decode_frame(dname, bindata),
                                 timestamp, dname, decoders_version))
    return json_obj


def parse_hex_frame(dname, params, json_obj):
    """
    Parser for hex frames
    """
    if params.hframe is not None:
        frame = re.sub('["\r\n]', '', params.hframe)
        bindata = binascii.unhexlify(frame)
        if params.tmstamp is not None:
            timestamp = params.tmstamp
        else:
            timestamp = datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M:%S')
        json_obj.append(
            create_point(decode_frame.decode_frame(dname, bindata), timestamp,
                         dname, decoders_version))
    return json_obj


def decode_multiple(dname, params, verbose=False):
    """
    Functional code goes here:
    """
    check_valid_format(params)
    json_obj = []
    if params.fname is not None:
        if verbose is True:
            print(f'Input file is: {params.fname}', file=sys.stderr)
    parse_csv_file(dname, params, json_obj, verbose)
    parse_bin_file(dname, params, json_obj)
    parse_hex_frame(dname, params, json_obj)
    return json.dumps(json_obj, indent=4, sort_keys=False)
